#!/usr/bin/env python
# -*- coding: utf-8 -*-

"""
test_quickbooks-py
----------------------------------

Tests for `quickbooks-py` module.
"""

import unittest

from quickbooks-py import quickbooks-py


class TestQuickbooks-py(unittest.TestCase):

    def setUp(self):
        pass

    def test_something(self):
        pass

    def tearDown(self):
        pass
